# Step-by-step installation of Kubernetes 1.25+ on the MPCDF HPC Cloud

The procedure below can be used to deploy a production-ready Kubernetes cluster
on the MPCDF [HPC Cloud](https://docs.mpcdf.mpg.de/doc/computing/cloud/),
including "out-of-the-box" support for persistent storage and load balancers.
The resulting cluster is intended to be functionally equivalent to the
[templatized version](../README.md).

*Values in `$ALL_CAPS` should be customized before running the command.*

*Instead of using the OpenStack CLI you can also use the
[dashboard](https://hpccloud.mpcdf.mpg.de/).*

## Variables
Set the following variables to your preferences:
```bash
EXTERNAL_NETWORK="cloud-public"
CONTROL_PLANE_FLAVOR="mpcdf.medium"
KEYNAME="___"
CLUSTER_NAME="________" \
WORKER_FLAVOR="mpcdf.large"
NUM_WORKERS=2
KUBERNETES_VERSION=1.26
CLIENT_CIDR="e.g. 130.183.0.0/16"
```
This is meant to ease the use of the instructions below. You can, of course,
enter values in place of using the variables below.

The flavors above are what is recommended from the flavors made avaialble by
default. If you need other hardware configurations to back your cluster, please
get in touch with us at the [helpdesk](https://helpdesk.mpcdf.mpg.de).

## Depoying the network


### Create a private network

You will need to request access to an external network such as *cloud-public*,
which is reachable from the internet. Be sure to use the same value in later
steps.
```bash
openstack network create k8s-net --mtu 1500
openstack subnet create k8s-subnet \
    --network k8s-net \
    --subnet-range 192.168.0.0/24 \
    --dns-nameserver 130.183.9.32 \
    --dns-nameserver 130.183.1.21
openstack router create k8s-router --external-gateway "${EXTERNAL_NETWORK}"
openstack router add subnet k8s-router k8s-subnet
```


### Security groups (i.e. Firewall rules)

Opening the traffic in our private network is safe as access is managed through
the load balancer and (optionally) the SSH  gateway.
```sh
openstack security group create k8s-secgroup
openstack security group rule create k8s-secgroup \
    --remote-group k8s-secgroup \
    --description "Allow all internal traffic"
openstack security group create k8s-control-secgroup
openstack security group rule create k8s-control-secgroup \
    --remote-ip $CLIENT_CIDR \
    --protocol tcp \
    --dst-port 22 \
    --description "Allow client SSH traffic"
openstack security group rule create k8s-control-secgroup \
    --remote-ip $CLIENT_CIDR \
    --protocol tcp \
    --dst-port 6443 \
    --description "Allow client k8s API traffic"
```


## Bootstrapping kubernetes


### The first control plane node


#### Instantiation

Edit the script `../common-config.sh`. You will need to set the two variables
`CONTAINERD_VERSION` and `KUBERNETES_VERSION`, which are provided by HEAT when
using the template method. Then use the script as user data for the controller
and worker nodes you create:
```bash
openstack server create k8s-control-plane \
    --image "Debian 11" \
    --flavor $CONTROL_PLANE_FLAVOR \
    --network k8s-net \
    --security-group k8s-secgroup \
    --security-group k8s-control-secgroup \
    --key-name $KEYNAME \
    --user-data ../common-config.sh
openstack floating ip create cloud-public
openstack server add floating ip k8s-control-plane FLOATING_IP_ADDRESS
```

Wait a moment for the node to come up and run the base configuration script.
Once the control plane node is up you can connect to it using the floating IP
and use it as an SSH gateway to the workers.


#### Configuration

Use the provided `kubeadm.yaml` and fill in you *cluster name* and the *floating
IP or DNS name of the load balancer*. Copy the resulting configuration to the
control plane using scp.

The ssh into the control plane and initialize kubernetes. *Note:* that you will
land on a node based on your source IP. If you want to start configuring another
control plane node first, scp/ssh over to that one.
```bash
ssh ${CONTROL_PLANE_IP} -l root
kubeadm init --upload-certs --config kubeadm.yaml
```

You will be given a command to add control plane nodes to the cluster:

```bash
kubeadm join $CLUSTER_NAME:6443 --token ___ \
	--discovery-token-ca-cert-hash ___ \
	--control-plane --certificate-key ___
```

And another to add workers to the cluster:
```bash
kubeadm join $CLUSTER_NAME:6443 --token ___ \
	--discovery-token-ca-cert-hash ___
```

You can now also grab the kubectl admin configuration file from
`/etc/kubernetes/admin.conf`. Set `kubectl` up to use it, more information is at
the [kubernetes docs](https://kubernetes.io/docs/tasks/tools/).


### Adding worker nodes

Now it is time to add some worker nodes. Make a second copy of
`../common-config.sh`, calling it, for example, `worker-config.sh`. Add the
command to join worker nodes to the cluster at the bottom. Use this new file
as the user data for the worker nodes:

```bash
openstack server create k8s-worker \
    --image "Debian 11" \
    --flavor $WORKER_FLAVOR \
    --network k8s-net \
    --security-group k8s-secgroup \
    --key-name $KEYNAME \
    --user-data worker-config.sh \
    --min $NUM_WORKERS \
    --max $NUM_WORKERS
```

Once again it will take a few minutes for the machines to come up and appear in
the cluster. You will be able to see them with:
```bash
kubectl get nodes
```

The output should look something like this:
```
NAME                  STATUS   ROLES           AGE     VERSION
k8s-control-plane   Ready    control-plane   32m   v1.26.6
k8s-worker-1        Ready    <none>          22s   v1.26.6
k8s-worker-2        Ready    <none>          23s   v1.26.6
```

# Configuring the kubernetes
## Container networking
We have tested using an overlay network (VXLAN). If you have high network
demands you may want to look into a more performant solution:
```bash
curl https://raw.githubusercontent.com/projectcalico/calico/v3.25.0/manifests/calico-vxlan.yaml \
    | sed 's/CrossSubnet/Always/' \
    | kubectl apply -f -
```

### Plug Kubernetes into OpenStack
You can use the OpenStack cloud provider
[integration](https://github.com/kubernetes/cloud-provider-openstack) to
provision persistent storage and load balancers for external access to your
kubernetes. You will need to create an openstack application credential for
kubernetes:
```bash
openstack application credential create k8s-cloud-provider
```
You will also need the subnet your workers reside on:
```bash
openstack subnet show k8s-subnet -f value -c id
```
and the identity of the external network you are using:
```bash
openstack network show $EXTERNAL_NETWORK -f value -c id
```

Fill the application credentials and network IDs into the provided `cloud.conf`,
the make it avaialble on your kubernetes as a secret:
```bash
kubectl create secret -n kube-system generic cloud-config --from-file=cloud.conf
```

We use the release of the
[cloud-provider-openstack](https://github.com/kubernetes/cloud-provider-openstack/)
corresponding to the kubernetes version:
```bash
git clone \
    --single-branch \
    --no-tags \
    --depth 1 \
    -b "release-$KUBERNETES_VERSION" \
    https://github.com/kubernetes/cloud-provider-openstack.git
rm cloud-provider-openstack/manifests/cinder-csi-plugin/csi-secret-cinderplugin.yaml

kubectl apply -f cloud-provider-openstack/manifests/controller-manager/cloud-controller-manager-roles.yaml
kubectl apply -f cloud-provider-openstack/manifests/controller-manager/cloud-controller-manager-role-bindings.yaml
kubectl apply -f cloud-provider-openstack/manifests/controller-manager/openstack-cloud-controller-manager-ds.yaml
kubectl apply -f cloud-provider-openstack/manifests/cinder-csi-plugin
kubectl apply -f cinder.yaml
```
